#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest


from chaoswm.actions import \
    add_mappings,           \
    populate_from_dir,      \
    delete_mappings,        \
    global_fixed_delay,     \
    global_random_delay,    \
    fixed_delay,            \
    random_delay,           \
    chunked_dribble_delay,  \
    down,                   \
    up,                     \
    reset

from chaoswm.probes import  \
    mappings

from chaoswm.utils import can_connect_to, get_wm_params

from http.client import HTTPConnection
import logging
HTTPConnection.debuglevel = 1
logging.basicConfig()
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True


@unittest.skipIf(can_connect_to("localhost", 8080) is False, "Test skipped: wiremock server not running")
class TestActions( unittest.TestCase):

    def test_wm_server_void_config(self):
        self.assertIsNone(get_wm_params({}))

    def test_wm_server_host_and_port(self):
        params = get_wm_params({
            "wiremock": {
                "host": "localhost",
                "port": 8080,
                "timeout": 10
            }
        })
        self.assertEqual(params['url'], "http://localhost:8080")
        self.assertEqual(params['timeout'], 10)

    def test_wm_server_host_and_port_and_context_path(self):
            params = get_wm_params({
                "wiremock": {
                    "host": "localhost",
                    "port": 8080,
                    "contextPath": "/contextPath",
                    "timeout": 10
                }
            })
            self.assertEqual(params['url'], "http://localhost:8080/contextPath")
            self.assertEqual(params['timeout'], 10)

    def test_global_fixed_delay(self):
        self.assertEqual(global_fixed_delay(
            10,
            {
                "wiremock":{
                    "host":"localhost",
                    "port": 8080,
                }
            }), 1)

    def test_global_random_delay(self):
        self.assertEqual(global_random_delay(
            {
                "type": "lognormal",
                "median": 20,
                "sigma": 0.1
            },
            {
                "wiremock":{
                    "host":"localhost",
                    "port": 8080,
                }
            }), 1)


    def test_add_mappings(self):
        ids = add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080,
           },
        })

        delete_mappings([{
                "url":"/some/thing", 
                "method":"GET"
            }],
            {
                "wiremock":{
                   "host":"localhost",
                   "port": 8080,
              }
            })

        self.assertTrue(isinstance(ids[0], str))
        self.assertTrue(ids[0] != -1)


    def test_populate_from_dir(self):

        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)

        ids = populate_from_dir(dir='./tests/mappings', configuration={
           "wiremock":{
               "host":"localhost",
               "port": 8080,
           },
        })
        self.assertEqual(len(ids), 2)

        reset({
                "wiremock":{
                   "host":"localhost",
                   "port": 8080,
              }
        })
        self.assertTrue(isinstance(ids[0], str))
        self.assertTrue(ids[0] != -1)


    def test_delete_mappings(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        ids_deleted=delete_mappings([{
                "url":"/some/thing", 
                "method":"GET"
            }],
            {
                "wiremock":{
                   "host":"localhost",
                   "port": 8080
              }
            }
        )
        self.assertTrue(isinstance(ids_deleted[0], str))
        self.assertEqual(ids_added[0], ids_deleted[0])


    def test_fixed_delay(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        delayed = fixed_delay(
            filter=[{
                "url":"/some/thing",
                "method":"GET"
            }],
            fixedDelayMilliseconds=1000,
            configuration={
               "wiremock":{
                   "host":"localhost",
                   "port": 8080
               },
            })
        self.assertEqual(delayed[0]['id'], ids_added[0])
        ids_deleted=delete_mappings([{
                "url":"/some/thing", 
                "method":"GET"
            }],
            {
                "wiremock":{
                   "host":"localhost",
                   "port": 8080
              }
            }
        )
        self.assertTrue(isinstance(ids_deleted[0], str))
        self.assertEqual(ids_added[0], ids_deleted[0])


    def test_random_delay(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        delayed = random_delay(
            filter=[{
                "url":"/some/thing",
                "method":"GET"
            }],
            delayDistribution={
                "type": "lognormal",
                "median": 80,
                "sigma": 0.4
            },
            configuration={
               "wiremock":{
                   "host":"localhost",
                   "port": 8080
               },
            })
        self.assertEqual(delayed[0]['id'], ids_added[0])
        ids_deleted=delete_mappings([{
                "url":"/some/thing", 
                "method":"GET"
            }],
            {
                "wiremock":{
                   "host":"localhost",
                   "port": 8080
              }
            }
        )
        self.assertTrue(isinstance(ids_deleted[0], str))
        self.assertEqual(ids_added[0], ids_deleted[0])


    def test_chunked_dribble_delay(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        delayed = chunked_dribble_delay(
            filter=[{
                "url":"/some/thing",
                "method":"GET"
            }],
            chunkedDribbleDelay={
                "numberOfChunks": 5,
                "totalDuration": 1000
            },
            configuration={
               "wiremock":{
                   "host":"localhost",
                   "port": 8080
               },
            })

        self.assertEqual(delayed[0]['id'], ids_added[0])
        self.assertEqual(delayed[0]['response']['chunkedDribbleDelay']['numberOfChunks'], 5)
        ids_deleted=delete_mappings([{
                "url":"/some/thing", 
                "method":"GET"
            }],
            {
                "wiremock":{
                   "host":"localhost",
                   "port": 8080
              }
            }
        )
        self.assertTrue(isinstance(ids_deleted[0], str))
        self.assertEqual(ids_added[0], ids_deleted[0])


    def test_down(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        delayed = down(
            filter=[{
                "method": "GET",
                "url": "/some/thing"
            }],
            configuration={
                "wiremock":{
                    "host":"localhost",
                    "port": 8080,
                    "defaults": {
                        "down": {
                            "numberOfChunks": 10,
                            "totalDuration": 4000
                        }
                    }
                },
            })
        self.assertEqual(delayed[0]['id'], ids_added[0])
        self.assertEqual(delayed[0]['response']['chunkedDribbleDelay']['numberOfChunks'], 10)


    def test_up(self):
        self.assertEqual(reset(
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        }), 1)
        ids_added=add_mappings([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }],
        {
           "wiremock":{
               "host":"localhost",
               "port": 8080
           },
        })
        delayed = down(
            filter=[{
                "method": "GET",
                "url": "/some/thing"
            }],
            configuration={
                "wiremock":{
                    "host":"localhost",
                    "port": 8080,
                    "defaults": {
                        "down": {
                            "numberOfChunks": 10,
                            "totalDuration": 4000
                        }
                    }
                },
            })
            
        self.assertEqual(delayed[0]['id'], ids_added[0])

        reset_ids = up(
            filter=[{
                "method": "GET",
                "url": "/some/thing"
            }],
            configuration={
                "wiremock":{
                    "host":"localhost",
                    "port": 8080
                }
            })
        m = mappings({
            "wiremock":{
                "host":"localhost",
                "port": 8080
            }
        })
        self.assertTrue("chunkedDribbleDelay" not in m[0]["response"])



# -*- coding: utf-8 -*-
"""Top-level package for chaoswm."""
from typing import List
from logzero import logger

from chaoslib.discovery.discover import discover_actions, discover_probes, \
    initialize_discovery_result
from chaoslib.exceptions import DiscoveryFailed
from chaoslib.types import Discovery, DiscoveredActivities, \
    DiscoveredSystemInfo, Secrets

name = "chaoswm"
__author__ = """Marco Masetti"""
__email__ = 'marco.masetti@sky.uk'
__version__ = '0.1.9'
__all__ = ["discover", "__version__"]


def discover(discover_system: bool = True) -> Discovery:
    logger.info("Discovering capabilities from chaostoolkit-wiremock")

    discovery = initialize_discovery_result(
        "chaostoolkit-wiremock", __version__, "wiremock")
    discovery["activities"].extend(load_exported_activities())
    return discovery


###############################################################################
# Private functions
###############################################################################
def load_exported_activities() -> List[DiscoveredActivities]:
    """
    Extract metadata from actions and probes exposed by this extension.
    """
    activities = []
    activities.extend(discover_actions("chaoswm.actions"))
    return activities



# -*- coding: utf-8 -*-
from logzero import logger

from chaoslib.types import Configuration
from .driver import Wiremock
from .utils import get_wm_params, check_configuration

__all__ = [
        "add_mappings",
        "populate_from_dir",
        "delete_mappings",
        "global_fixed_delay",
        "global_random_delay",
        "fixed_delay"
        "random_delay",
        "chunked_dribble_delay",
        "down",
        "up",
        "reset"
]


def add_mappings(mappings: list=[], configuration: Configuration={}):
    """ adds more mappings to wiremock
    returns the list of ids of the mappings added
    """
    if (not check_configuration(configuration)):
        return []

    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])
    return w.populate(mappings)


def populate_from_dir(dir: str=".", configuration: Configuration={}):
    """ adds all mappings found in the passed folder
    returns the list of ids of the mappings added
    """
    if (not check_configuration(configuration)):
        return []

    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])
    return w.populate_from_dir(dir)


def delete_mappings(filter: list=[], configuration: Configuration={}):
    """ deletes a list of mappings
    returns the list of ids of the mappings deleted
    """
    if (not check_configuration(configuration)):
        return []

    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    ids = []
    for f in filter:
        mapping = w.mapping_by_request(f)
        if ("id" not in mapping):
            logger.error("Error: mapping {} {} not found".format(method, url))
            continue
        ids.append(w.delete_mapping(mapping["id"]))
    return ids


def down(filter: list=[], configuration: Configuration={}):
    """ set a list of services down
    more correctly it adds a chunked dribble delay to the mapping
    as defined in the configuration section (or action attributes)
    Returns the list of delayed mappings
    """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    conf = configuration.get('wiremock', {})
    if ('defaults' not in conf):
        logger.error("down defaults not specified in config")
        return -1

    defaults = conf.get('defaults',{})
    if ('down' not in defaults):
        logger.error("down defaults not specified in config")
        return -1

    delayed = []
    for f in filter:
        delayed.append(w.chunked_dribble_delay(f, defaults['down']))
    return delayed


def global_fixed_delay(fixedDelay: int=0, configuration: Configuration={}):
    """ add a fixed delay to all mappings """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])
    return w.global_fixed_delay(fixedDelay)


def global_random_delay(delayDistribution: dict={}, configuration: Configuration={}):
    """ adds a random delay to all mappings """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])
    return w.global_random_delay(delayDistribution)

    
def fixed_delay(filter: list, fixedDelayMilliseconds: int, configuration: Configuration={}):
    """ adds a fixed delay to a list of mappings """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    updated=[]
    for f in filter:
        updated.append(w.fixed_delay(f, fixedDelayMilliseconds))
    return updated


def random_delay(filter: list=[], delayDistribution: dict={}, configuration: Configuration={}):
    """adds a random delay to a list of mapppings"""
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    updated=[]
    for f in filter:
        updated.append(w.random_delay(f, delayDistribution))
    return updated


def chunked_dribble_delay(filter: list=[], chunkedDribbleDelay: dict={}, configuration: Configuration={}):
    """adds a chunked dribble delay to a list of mappings"""
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    updated=[]
    for f in filter:
        updated.append(w.chunked_dribble_delay(f, chunkedDribbleDelay))
    return updated


def up(filter: list=[], configuration: Configuration={}):
    """ deletes all delays connected with a list of mappings """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    return w.up(filter)

def reset(configuration: Configuration={}):
    """ resets the wiremock server: deletes all mappings! """
    params = get_wm_params(configuration)
    w = Wiremock(url=params['url'], timeout=params['timeout'])

    return w.reset()



# -*- coding: utf-8 -*-
from logzero import logger

from chaoslib.types import Configuration
from sky_wiremock.sky_wiremock import Wiremock, ConnectionError
from .utils import get_wm_params, check_configuration

__all__ = [
        "mappings",
        "server_running"
]


def server_running(c: Configuration={}):
    if (not check_configuration(c)):
        return []
    try:
        params = get_wm_params(c)
        w = Wiremock(url=params['url'], timeout=params['timeout'])
        return 1
    except ConnectionError:
        logger.error("Error: Wiremock server not running")
        return 0


def mappings(c: Configuration={}):
    if (not check_configuration(c)):
        return []
    try:
        params = get_wm_params(c)
        w = Wiremock(url=params['url'], timeout=params['timeout'])
        return w.mappings()
    except ConnectionError:
        logger.error("Error connecting to Wiremock server")
        return None




import unittest
import sky_wiremock
from sky_wiremock.sky_wiremock import Wiremock
from sky_wiremock.utils import can_connect_to


@unittest.skipUnless(can_connect_to("localhost",8080), "Wiremock server not found")
class TestSkyWiremock(unittest.TestCase):

    def test_add_mapping(self):
        w = Wiremock( host="localhost", port=8080, timeout=2)
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        self.assertTrue(isinstance(id, str))
        self.assertTrue(id != -1)


    def test_update_mapping(self):
        w = Wiremock( host="localhost", port=8080, timeout=2)
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
#         import pdb; pdb.set_trace()
        updated = w.update_mapping(
            id=id,
            mapping={
                "request": {
                    "method": "GET",
                    "url": "/some/thing"
                },
                "response": {
                    "status": 200,
                    "body": "Hello world!",
                    "fixedDelayMilliseconds":10000,
                    "headers": {
                        "Content-Type": "text/plain"
                    }
                } 
            }
        )
        self.assertTrue(isinstance(updated["id"], str))
        self.assertEqual(id, updated["id"])


    def test_populate(self):
        w = Wiremock( host="localhost", port=8080)
        ids = w.populate([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            }
        },{
            "request": {
                "method": "GET",
                "url": "/some/thing/else"
            },
                "response": {
                "status": 200,
                "body": "Hello world again!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            }
        }])
        # the method should return a list with all the new mapping ids
        # or -1 in case of errors
        self.assertTrue(isinstance(ids, list))
        self.assertEqual(len(ids), 2)


    def test_populate_from_dir(self):
#         import pdb; pdb.set_trace()
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        self.assertEqual(len(w.populate_from_dir('./tests/mappings')), 2)
        mappings = w.mappings()
        self.assertEqual(len(mappings), 2)

    def test_delete_mapping(self):
        w = Wiremock( host="localhost", port=8080)
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        ret = w.delete_mapping(id)
        self.assertTrue(isinstance(ret, str))
        self.assertTrue(ret != -1)

    def test_mappings(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mappings = w.mappings()
        self.assertTrue(isinstance(mappings, list))
        self.assertEqual(len(mappings), 1)


    def test_mapping_by_id(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mapping = w.mapping_by_id(id)
        self.assertTrue(isinstance(mapping, dict))
        self.assertEqual(mapping["id"], id)

    def test_mapping_by_url_and_method(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mapping = w.mapping_by_url_and_method("/some/thing/to/delete", "GET")
        self.assertTrue(isinstance(mapping, dict))
        self.assertEqual(mapping["id"], id)

    def test_mapping_by_request(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        self.assertEqual(len(w.populate_from_dir('./tests/mappings')), 2)
#         import pdb; pdb.set_trace()
        filter = {
            "method": "GET",
            "url": "/epg",
            "queryParameters": {"episode_title": {"matches": "([a-z]*)"}}
        }
        mapping = w.mapping_by_request(filter)
        self.assertEqual(mapping['request'], filter)


    def test_mapping_by_request_exact_match(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        self.assertEqual(len(w.populate_from_dir('./tests/mappings')), 2)
#         import pdb; pdb.set_trace()
        filter = {
            "method": "GET",
            "url": "/epg"
        }
        mapping = w.mapping_by_request_exact_match(filter)
        self.assertEqual(mapping['request'], filter)


    def test_fixed_delay(self):
#         import pdb; pdb.set_trace()
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/delay"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed = w.fixed_delay(mapping['request'], 10000)
        self.assertTrue(isinstance(delayed, dict))
        self.assertTrue(isinstance(delayed["id"], str))
        self.assertEqual(delayed["id"], id)


    def test_global_fixed_delay(self):
        w = Wiremock( host="localhost", port=8080)
        ret = w.global_fixed_delay(300)
        self.assertEqual(ret, 1)


    def test_random_delay(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed = w.random_delay(
        filter=mapping['request'],
        delayDistribution={
            "type": "lognormal",
            "median": 80,
            "sigma": 0.4
        })
        self.assertTrue(isinstance(delayed['id'], str))
        self.assertEqual(delayed['id'], id)


    def test_global_random_delay(self):
        w = Wiremock( host="localhost", port=8080)
        ret = w.global_random_delay({
            "type": "lognormal",
            "median": 90,
            "sigma": 0.1
        })
        self.assertEqual(ret, 1)


    def test_chunked_dribble_delay(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed = w.chunked_dribble_delay(mapping["request"], {
            "numberOfChunks": 5,
            "totalDuration": 1000
        })
        self.assertTrue(isinstance(delayed, dict))
        self.assertTrue(isinstance(delayed["id"], str))
        self.assertEqual(delayed["id"], id)

        map = w.mapping_by_id(delayed["id"])
        self.assertEqual(map["response"]["chunkedDribbleDelay"]["totalDuration"], 1000)

    def test_up(self):
        w = Wiremock( host="localhost", port=8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                },
                "chunkedDribbleDelay":{
                    "numberOfChunks": 5,
                    "totalDuration": 1000
                }
            } 
        }
        id = w.add_mapping(mapping)

        stored_mapping = w.mapping_by_id(id)
        self.assertEqual(stored_mapping["id"], id)
        self.assertEqual(stored_mapping["response"]["chunkedDribbleDelay"]["numberOfChunks"], 5)
        ids_up = w.up([{
            "method": "GET",
            "url": "/some/thing/to/delete"
        }])

        up_mapping = w.mapping_by_id(ids_up[0])
        self.assertFalse("chunkedDribbleDelay" in up_mapping["response"])

    def test_reset(self):
        w = Wiremock( host="localhost", port=8080)
        self.assertEqual(w.reset(), 1)



import socket
from logzero import logger

from contextlib import closing


def can_connect_to(host, port):
    """ Test a connection to a host/port """

    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        return bool(sock.connect_ex((host, port)) == 0)


def get_wm_params(c: dict):
    wm_conf = c.get("wiremock", {})

    host = wm_conf.get("host", None)
    port = wm_conf.get("port", None)
    context_path = wm_conf.get("contextPath", "")
    timeout = wm_conf.get("timeout", 1)

    url = ""

    if host and port:
        url = "http://{}:{}{}".format(host, port, context_path)

    if not url:
        logger.error("ERROR: no configuration params to set WM server url")
        return None

    return {"url": url, "timeout": timeout}


def check_configuration(c: dict = {}):
    if "wiremock" not in c:
        logger.error("Error: wiremock key not found in configuration section")
        return -1
    return 1



# -*- coding: utf-8 -*-
"""Main module.

This module implements a few functions to handle mappings on a wiremock server.
It is meant not to be complete and to be the most transparent possible, 
in contrast with the official wiremock driver. For example there is no
validation of the payloads.

"""

import glob
import os
import sys
import json
import requests
from logzero import logger
from typing import Any, Dict, Union
from .utils import can_connect_to


class ConnectionError(Exception):
    pass


class Wiremock(object):

    def __init__(self, host=None, port=None, url=None, timeout=1):

        if (host and port):
            url="http://{}:{}".format(host,port)
        self.base_url  = "{}/__admin".format(url)
        self.mappings_url = "{}/{}".format(self.base_url, "mappings")
        self.settings_url = "{}/{}".format(self.base_url, "settings")
        self.reset_url    = "{}/{}".format(self.base_url, "reset")
        self.timeout=timeout
        self.headers = {
            'Accept': 'application/json', 
            'Content-Type': 'application/json'
        }

        if ((host and port) and can_connect_to(host, port) == False):
            raise ConnectionError("Error: wiremock server not found")


    def mappings(self):
        """
        retrieves all mappings
        returns the array of mappings found
        """
        r = requests.get(
            self.mappings_url,
            headers=self.headers,
            timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("[mappings]:Error retrieving mappings: {}".format(r.text())) 
            return []
        else:
            res = r.json()
            return res["mappings"]


    def mapping_by_id(self, id):
        r = requests.get(
            "{}/{}".format(self.mappings_url, id),
            headers=self.headers,
            timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("[mapping_by_id]:Error retrieving mapping: {}".format(r.text())) 
            return -1
        else:
            return r.json()


    def mapping_by_url_and_method(self, url, method):
        mappings = self.mappings()
        for mapping in mappings:
            if ((mapping["request"]["url"] == url) and
                (mapping["request"]["method"] == method)):
                return mapping
        return {}


    def mapping_by_request(self, request: dict):
        num_filter_keys = len(request.keys())
        mappings = self.mappings()
        for mapping in mappings:
            count=0
            inter = mapping['request'].keys() & request.keys()
            if ( len(inter) != num_filter_keys):
                continue
            for k in request.keys():
                if (mapping['request'][k] == request[k]):
                    count += 1
            if (count == num_filter_keys):
                return mapping
        return None


    def mapping_by_request_exact_match(self, request: dict):
        num_filter_keys = len(request.keys())
        mappings = self.mappings()
        for mapping in mappings:
            if (mapping['request'] == request):
                return mapping
        return None


    def populate(self, mappings): 
        """ Populate: adds all passed mappings 
            Returns the list of ids of mappings created
        """
        if (isinstance(mappings, list) == False):
            logger.error("[populate]:ERROR: mappings should be a list")
            return -1

        ids = []
        for mapping in mappings:
            ret = self.add_mapping(mapping)
            if (ret != -1):
                ids.append(ret)
            else:
                logger.error("[populate]:ERROR adding a mapping")
                return -1
            
        return ids


    def populate_from_dir(self, dir):
        """ reads all json files in a directory and adds all mappings
            Returns the list of ids of mappings created
            or None in case of errors
        """
        logger.info("populate_from_dir")

        if (not os.path.exists(dir)):
            logger.error("Error: directory {} does not exists".format(dir))
            return None

        ids = []
        for filename in glob.glob(os.path.join(dir, '*.json')):
            logger.info("Importing {}".format(filename))
            with open(filename) as f:
                mapping = json.load(f)
                ids.append(self.add_mapping(mapping))
        return ids


    def update_mapping(self, id: str="", mapping: dict = {}):
        """ updates the mapping pointed by id with new mapping """
        r = requests.put(
                "{}/{}".format(self.mappings_url, id),
                headers=self.headers,
                data=json.dumps(mapping),
                timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("Error updating a mapping: "+r.text)
            return None
        else:
            return r.json()


    def add_mapping(self, mapping: Dict):
        """ add_mapping: add a mapping passed as attribute """
        r = requests.post(
                self.mappings_url,
                headers=self.headers,
                data=json.dumps(mapping),
                timeout=self.timeout)
        if (r.status_code != 201):
            logger.error("Error creating a mapping: "+r.text)
            return -1
        else:
            res = r.json()
            return res['id']
    

    def delete_mapping(self, id: str):
        r = requests.delete(
                "{}/{}".format(self.mappings_url, id),
                timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("Error deleting mapping {}: {}".format(id, r.text))
            return -1
        else:
            return id

    def delete_all_mappings(self):
        mappings = self.mappings()
        for mapping in mappings:
            self.delete_mapping(mapping["id"])
        return len(mappings)


    def fixed_delay(self, request, fixedDelayMilliseconds=0):
        """ 
        updates the mapping adding a fixed delay 
        returns the updated mapping or none in case of errors
        """
        mapping_found = self.mapping_by_request_exact_match(request)

        if ("id" not in mapping_found):
            logger.error("[fixed_delay]: Error retrieving mapping")
            return None

        mapping_found["response"]["fixedDelayMilliseconds"] = fixedDelayMilliseconds
        return self.update_mapping(mapping_found["id"], mapping_found)


    def global_fixed_delay(self, fixedDelay):
        r = requests.post(
                self.settings_url,
                headers=self.headers,
                data=json.dumps({"fixedDelay": fixedDelay}),
                timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("[global_fixed_delay]: Error setting delay: {}".format(r.text))
            return -1
        else:
            return 1
        

    def random_delay(self, filter, delayDistribution):
        """
        Updates the mapping adding a random delay
        returns the updated mapping or none in case of errors
        """
        if ( not isinstance(delayDistribution, dict)):
            logger.error("[random_delay]: parameter has to be a dictionary");

        mapping_found = self.mapping_by_request_exact_match(filter)

        if ("id" not in mapping_found):
            logger.error("[fixed_delay]: Error retrieving mapping")
            return None

        mapping_found["response"]["delayDistribution"] = delayDistribution
        return self.update_mapping(mapping_found["id"], mapping_found)

    def global_random_delay(self, delayDistribution):
        if ( not isinstance(delayDistribution, dict)):
            logger.error("[global_random_delay]: parameter has to be a dictionary");
        r = requests.post(
                self.settings_url,
                headers=self.headers,
                data=json.dumps({"delayDistribution": delayDistribution}),
                timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("[global_random_delay]: Error setting delay: {}".format(r.text))
            return -1
        else:
            return 1

    def chunked_dribble_delay(self, filter, chunkedDribbleDelay):
        """
        Adds a delay to the passed mapping
        returns the updated mapping or non in case of errors
        """
        if ( not isinstance(chunkedDribbleDelay, dict)):
            logger.error("[chunked_dribble_delay]: parameter has to be a dictionary");
        if ( "numberOfChunks" not in chunkedDribbleDelay ):
            logger.error("chunked_dribble_delay]: attribute numberOfChunks not found in parameter")
            return None
        if ( "totalDuration" not in chunkedDribbleDelay ):
            logger.error("chunked_dribble_delay]: attribute totalDuration not found in parameter")
            return None

        mapping_found = self.mapping_by_request_exact_match(filter)

        if ("id" not in mapping_found):
            logger.error("[chunked_dribble_delay]: Error retrieving mapping")
            return None

        mapping_found["response"]["chunkedDribbleDelay"] = chunkedDribbleDelay
        return self.update_mapping(mapping_found["id"], mapping_found)

    def up(self, filter: list=[]):
        """ resets a list of mappings deleting all delays attached to them """
        ids = []
        for f in filter:
            mapping_found = self.mapping_by_request_exact_match(f)
            if (not mapping_found):
                next
            else:
                for key in ['fixedDelayMilliseconds', 'delayDistribution', 'chunkedDribbleDelay']:
                    if (key in mapping_found["response"]): del mapping_found["response"][key]

#                 self.delete_mapping(mapping_found["id"])
#                 priority = mapping_found.get('priority', 1)
                up_mapping = {
#                     "priority": priority,
                    "request": mapping_found["request"],
                    "response": mapping_found["response"]
                }
                ids.append(self.add_mapping(up_mapping))
        return ids


    def reset(self):
        r = requests.post(
                self.reset_url,
                headers=self.headers,
                timeout=self.timeout)
        if (r.status_code != 200):
            logger.error("Error resetting wiremock server "+r.text)
            return -1
        else:
            return 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
import os
import io


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = []
with io.open('requirements.txt') as f:
    requirements = [l.strip() for l in f if not l.startswith('#')]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

def get_version_from_package() -> str:
    path = os.path.join(os.path.dirname(__file__), "chaoswm/__init__.py")
    path = os.path.normpath(os.path.abspath(path))
    with open(path) as f:
        for line in f:
            if line.startswith("__version__"):
                token, version = line.split(" = ", 1)
                version = version.replace("'", "").strip()
                return version

setup(
    author="Marco Masetti",
    author_email='marco.masetti@sky.uk',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="chaostoolkit driver for wiremock",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='chaoswm',
    name='chaoswm',
    packages=find_packages(include=['chaoswm']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/sky-uk/chaostoolkit-wiremock',
    version=get_version_from_package(),
    zip_safe=False
)
